<?php
/**
 * FORGINGBLOCK
 */
use PrestaShop\PrestaShop\Core\Payment\PaymentOption; 

define("BTCPAY_VERSION", "0.0.1");
$autoloader_param = __DIR__ . '/lib/Bitpay/Autoloader.php';

if (!defined('_PS_VERSION_'))
    exit;

// Load up the BitPay library
if (true === file_exists($autoloader_param) &&
    true === is_readable($autoloader_param))
{
  if(false === class_exists("Bitpay\Autoloader")){
    require_once $autoloader_param;
    \Bitpay\Autoloader::register();
  }
} else {
    throw new \Exception('The Forgingblock PrestaShop payment plugin was not installed correctly or the files are corrupt. Please reinstall the plugin. If this message persists after a reinstall, contact the Forgingblock PrestaShop team through http://slack.btcpayserver.org with this message.');
}

// Exist for quirks in object serialization...
if (false === class_exists('Bitpay\PrivateKey')) {
    include_once(__DIR__ . '/lib/Bitpay/PrivateKey.php');
}

if (false === class_exists('Bitpay\PublicKey')) {
    include_once(__DIR__ . '/lib/Bitpay/PublicKey.php');
}

if (false === class_exists('Bitpay\Token')) {
    include_once(__DIR__ . '/lib/Bitpay/Token.php');
}


class Forgingblock extends PaymentModule
{
    public function __construct()
    {
        $this->name = 'forgingblock';
        $this->tab = 'payments_gateways';
        $this->version = '1.75';
        $this->author = 'Forgingblock';
        $this->need_instance = 1;
        $this->bootstrap = true;
		$this->ps_versions_compliancy = array('min' => '1.7.1.0', 'max' => _PS_VERSION_);
		$this->author = 'forgingblock';

        parent::__construct();

        $this->displayName = 'forgingblock';
        $this->description = $this->l('Secure payement with Forgingblock.');	

    }

	
	public function install() {

		if ( ! parent::install() ||
		     ! $this->registerHook( 'paymentOptions' ) ||
		     ! $this->registerHook( 'orderConfirmation' ) ||
		     ! Configuration::updateValue( 'FORGINGBLOCK_TITLE', 'Secure payment with forgingblock.' ) ||
		     ! Configuration::updateValue( 'FORGINGBLOCK_ALIAS', null ) 

		) {
			return false;
		}

		return true;

	}	
	
	public function uninstall() {
		if ( ! parent::uninstall() ||
		     ! Configuration::deleteByName( 'FORGINGBLOCK_TITLE' ) ||
		     ! Configuration::deleteByName( 'FORGINGBLOCK_ALIAS' ) 
		) {
			return false;
		}

		return true;
	}
	
	
	public function getContent() {
		$output = null;

		if ( Tools::isSubmit( 'submit' . $this->name ) ) {
			
			if ($_REQUEST['revoke']) {
				Configuration::updateValue('FORGINGBLOCK_KEY', '' );
				Configuration::updateValue('FORGINGBLOCK_PUB', '');
				Configuration::updateValue('FORGINGBLOCK_SIN', '');
				Configuration::updateValue('FORGINGBLOCK_TOKEN', '');		
				Configuration::updateValue('FORGINGBLOCK_LABEL', '' );				
				Configuration::updateValue('FORGINGBLOCK_URL', '' );					
				Configuration::updateValue('FORGINGBLOCK_PAIRCODE', '' );	
			}
			
			$urllist = array('https://api-demo.forgingblock.io', 'https://api.forgingblock.io');
			$forgingblock_title   = strval( Tools::getValue( 'FORGINGBLOCK_TITLE' ) );
			$forgingblock_baseurl  = strval( Tools::getValue( 'FORGINGBLOCK_BASEURL' ) );
			$forgingblock_parcode  = strval( Tools::getValue( 'FORGINGBLOCK_PAIRCODE' ) );	

			if ($forgingblock_parcode) {
				if (!preg_match('/^[a-zA-Z0-9]{7}$/', $forgingblock_parcode)) {
					$output .= $this->displayError( $this->l( 'Invalid Pairing Code' ) );
                	return $output . $this->displayForm();                
            	}				
				
				else 					
				{
					$parcodeact =  $this->update_btcpay_pair_code($forgingblock_parcode, $urllist[$forgingblock_baseurl]);
					if ($parcodeact) {
						$output .= $this->displayError( $this->l( $parcodeact ) );
						return $output . $this->displayForm();                
					}
				}
				
				
			}

			if ( ! $forgingblock_title
			     || empty( $forgingblock_title )
			     || ! Validate::isGenericName( $forgingblock_title )			     			     
			) {
				$output .= $this->displayError( $this->l( 'Invalid Configuration.' ) );
			} else {
				Configuration::updateValue( 'FORGINGBLOCK_TITLE', $forgingblock_title );
				Configuration::updateValue( 'FORGINGBLOCK_BASEURL', $forgingblock_baseurl );							
				Configuration::updateValue( 'FORGINGBLOCK_PAIRCODE', $forgingblock_parcode );
				
				Configuration::updateValue( 'FORGINGBLOCK_TRANSSPD', strval( Tools::getValue( 'FORGINGBLOCK_TRANSSPD' ) ));
				Configuration::updateValue( 'FORGINGBLOCK_NEWST', strval( Tools::getValue( 'FORGINGBLOCK_NEWST' ) ));
				Configuration::updateValue( 'FORGINGBLOCK_PAIDST', strval( Tools::getValue( 'FORGINGBLOCK_PAIDST' ) ));				
				Configuration::updateValue( 'FORGINGBLOCK_CONFIRMRFST', strval( Tools::getValue( 'FORGINGBLOCK_CONFIRMRFST' ) ));
				Configuration::updateValue( 'FORGINGBLOCK_COMPLETEST', strval( Tools::getValue( 'FORGINGBLOCK_COMPLETEST' ) ));
				Configuration::updateValue( 'FORGINGBLOCK_INVALIDST', strval( Tools::getValue( 'FORGINGBLOCK_INVALIDST' ) ));
				Configuration::updateValue( 'FORGINGBLOCK_EXPIREDST', strval( Tools::getValue( 'FORGINGBLOCK_EXPIREDST' ) ));				
				Configuration::updateValue( 'FORGINGBLOCK_PAIDEXPIREDST', strval( Tools::getValue( 'FORGINGBLOCK_PAIDEXPIREDST' ) ));
				Configuration::updateValue( 'FORGINGBLOCK_PARTEXPIREDST', strval( Tools::getValue( 'FORGINGBLOCK_PARTEXPIREDST' ) ));				
				
				
				$output .= $this->displayConfirmation( $this->l( 'Settings updated.' ) );
			}
		}

		return $output . $this->displayForm();
	}
	
	public function update_btcpay_pair_code($pairing_code, $url)
    {
        	

            // Generate Private Key
            $key = new \Bitpay\PrivateKey();

            if (true === empty($key)) {
                throw new \Exception('The Forgingblock PrestaShop payment plugin was called to process a pairing code but could not instantiate a PrivateKey object. Cannot continue!');
            }

            $key->generate();

            // Generate Public Key
            $pub = new \Bitpay\PublicKey();

            if (true === empty($pub)) {
                throw new \Exception('The Forgingblock PrestaShop payment plugin was called to process a pairing code but could not instantiate a PublicKey object. Cannot continue!');
            }

            $pub->setPrivateKey($key);
            $pub->generate();

            // Get SIN Format
            $sin = new \Bitpay\SinKey();

            if (true === empty($sin)) {
                throw new \Exception('The Forgingblock PrestaShop payment plugin was called to process a pairing code but could not instantiate a SinKey object. Cannot continue!');
            }

            $sin->setPublicKey($pub);
            $sin->generate();

            // Create an API Client
            $client = new \Bitpay\Client\Client();

            if (true === empty($client)) {
                throw new \Exception('The Forgingblock PrestaShop payment plugin was called to process a pairing code but could not instantiate a Client object. Cannot continue!');
            }
            $client->setUri($url);
            $curlAdapter = new \Bitpay\Client\Adapter\CurlAdapter();

            if (true === empty($curlAdapter)) {
                throw new \Exception('The Forgingblock PrestaShop payment plugin was called to process a pairing code but could not instantiate a CurlAdapter object. Cannot continue!');
            }

            $client->setAdapter($curlAdapter);

            $client->setPrivateKey($key);
            $client->setPublicKey($pub);

            // Sanitize label
            
            $label = 'PrestaShop';

            try {
                $token = $client->createToken(
                    array(
                        'id'          => (string) $sin,
                        'pairingCode' => $pairing_code,
                        'label'       => $label,
                    )
                );
            } catch (\Exception $e) {                
                return $e->getMessage();
            }

			Configuration::updateValue('FORGINGBLOCK_KEY', $this->btcpay_encrypt($key) );
			Configuration::updateValue('FORGINGBLOCK_PUB', $this->btcpay_encrypt($pub) );
			Configuration::updateValue('FORGINGBLOCK_SIN', (string)$sin );		
			Configuration::updateValue('FORGINGBLOCK_TOKEN', $this->btcpay_encrypt($token) );		
			Configuration::updateValue('FORGINGBLOCK_LABEL', $label );				
			Configuration::updateValue('FORGINGBLOCK_URL', $url );	
            
        
    }
	
	public function btcpay_encrypt($data)
        {
            if (false === isset($data) || true === empty($data)) {
                throw new \Exception('The Forgingblock PrestaShop payment plugin was called to encrypt data but no data was passed!');
            }

            

            $openssl_ext = new \Bitpay\Crypto\OpenSSLExtension();
            $fingerprint = sha1(sha1(__DIR__));

            if (true === isset($fingerprint) &&
                true === isset($openssl_ext)  &&
                strlen($fingerprint) > 24)
            {
                $fingerprint = substr($fingerprint, 0, 24);

                if (false === isset($fingerprint) || true === empty($fingerprint)) {
                    throw new \Exception('The Forgingblock PrestaShop payment plugin was called to encrypt data but could not generate a fingerprint parameter!');
                }

                $encrypted = $openssl_ext->encrypt(base64_encode(serialize($data)), $fingerprint, '1234567890123456');

                if (true === empty($encrypted)) {
                    throw new \Exception('The Forgingblock PrestaShop payment plugin was called to encrypt a serialized object and failed!');
                }



                return $encrypted;
            } else {
                
                wp_die('Invalid server fingerprint generated');
            }
        }

	
	public function btcpay_decrypt($encrypted)
        {
            if (false === isset($encrypted) || true === empty($encrypted)) {
                throw new \Exception('The Forgingblock PrestaShop payment plugin was called to decrypt data but no data was passed!');
            }

            
         
            $openssl_ext = new \Bitpay\Crypto\OpenSSLExtension();

            $fingerprint = sha1(sha1(__DIR__));

            if (true === isset($fingerprint) &&
                true === isset($openssl_ext)  &&
                strlen($fingerprint) > 24)
            {
                $fingerprint = substr($fingerprint, 0, 24);

                if (false === isset($fingerprint) || true === empty($fingerprint)) {
                    throw new \Exception('The Forgingblock PrestaShop payment plugin was called to decrypt data but could not generate a fingerprint parameter!');
                }

                $decrypted = base64_decode($openssl_ext->decrypt($encrypted, $fingerprint, '1234567890123456'));

                // Strict base64 char check
                if (false === base64_decode($decrypted, true)) {
                    
                } else {
                    $decrypted = base64_decode($decrypted);
                }

                if (true === empty($decrypted)) {
                    throw new \Exception('The Forgingblock PrestaShop payment plugin was called to unserialize a decrypted object and failed! The decrypt function was called with "' . $encrypted . '"');
                }

                

                return unserialize($decrypted);
            } else {
                
                wp_die('Invalid server fingerprint generated');
            }
      
    }
	
	protected function displayForm() {
		// Get default language
		$default_lang = (int) Configuration::get( 'PS_LANG_DEFAULT' );
		
		$forgingblock_label = Configuration::get( 'FORGINGBLOCK_LABEL' ) ;		
		if (Configuration::get('FORGINGBLOCK_TOKEN' ) )
		 $forgingblock_token  =  $this->btcpay_decrypt(Configuration::get('FORGINGBLOCK_TOKEN' ) );
		
		$urls_list[] = array('id' => '0' , 'url' => 'https://api-demo.forgingblock.io');
		$urls_list[] = array('id' => '1' , 'url' => 'https://api.forgingblock.io');
		
		
		$transspd[] = array('id' => 'default' , 'name' => 'Keep store level configuration');		
		$transspd[] = array('id' => 'high' , 'name' => '0 confirmation on-chain');		
		$transspd[] = array('id' => 'medium' , 'name' => '1 confirmation on-chain');				
		$transspd[] = array('id' => 'low-medium' , 'name' => '2 confirmations on-chain');
		$transspd[] = array('id' => 'low' , 'name' => '6 confirmations on-chain');		

		// Init Fields form array
		$fields_form[1]['form'] = array(
			'legend' => array(
				'title' => $this->l( 'Settings' ),
			),
			'input'  => array(
				array(
					'type'     => 'text',
					'label'    => $this->l( 'Title' ),
					'name'     => 'FORGINGBLOCK_TITLE',
					'size'     => 50,
					'required' => true
				),
				array(
					'type' => 'select',
					'label'    => $this->l( 'Invoice pass to "confirmed" state after' ),
					'name'     => 'FORGINGBLOCK_TRANSSPD',					
						'options' => array(
                            'query' => $transspd,
                            'id' => 'id',
                            'name' => 'name',
                        ),
					'required' => true
				),	
				
			),
			
		);
		if ($forgingblock_token) {
			$fields_form[2]['form'] = array(
			'legend' => array(
				'title' => $this->l( 'Token' ),
			),
			'input'  => array(							
				array(
					'type'     => 'label',
					'label'    => $this->l( 'ID' ),					
					'desc'    => $forgingblock_token,					
				),
				
				array(
					'type'     => 'label',
					'label'    => $this->l( 'Label' ),					
					'desc'    => $forgingblock_label,					
				),				
					
			),
					
			'submit' => array(
				'title' => $this->l( 'Revoke' ),
				'name' => 'revoke',				
				'class' => 'btn btn-default pull-right'
			)
		);
			
		}
		else {
		$fields_form[2]['form'] = array(
			'legend' => array(
				'title' => $this->l( 'Token' ),
			),
			'input'  => array(				
				array(
					'type' => 'select',
					'label'    => $this->l( 'API URL' ),
					'name'     => 'FORGINGBLOCK_BASEURL',					
					'options' => array(
                            'query' => $urls_list,
                            'id' => 'id',
                            'name' => 'url',
                        ),
					'required' => true
				),
				array(
					'type'     => 'text',
					'label'    => $this->l( 'Pairing Code' ),
					'name'     => 'FORGINGBLOCK_PAIRCODE',
					'size'     => 32,
					'required' => true
				),
			),
			'submit' => array(
				'title' => $this->l( 'Pair' ),
				'class' => 'btn btn-default pull-right'
			)
		);
		}
		
		$states = 	OrderState::getOrderStates((int) Configuration::get('PS_LANG_DEFAULT'));
		foreach ($states as $state)				
			$OrderStates[$state['id_order_state']] = $state['name'];
		
		
		$fields_form[3]['form'] = array(
			'legend' => array(
				'title' => $this->l( 'Order Status' ),
			),
			'input'  => array(				
				array(
					'type' => 'select',
					'label'    => $this->l( 'New Order' ),
					'name'     => 'FORGINGBLOCK_NEWST',					
						'options' => array(
                            'query' => $states,
                            'id' => 'id_order_state',
                            'name' => 'name',
                        ),
					'required' => true
				),				
				
				array(
					'type' => 'select',
					'label'    => $this->l( 'Paid' ),
					'name'     => 'FORGINGBLOCK_PAIDST',										
					'options' => array(
                            'query' => $states,
                            'id' => 'id_order_state',
                            'name' => 'name',
                        ),					
					'required' => true
				),	
				
				array(
					'type' => 'select',
					'label'    => $this->l( 'Confirmed' ),
					'name'     => 'FORGINGBLOCK_CONFIRMRFST',										
					'options' => array(
                            'query' => $states,
                            'id' => 'id_order_state',
                            'name' => 'name',
                        ),					
					'required' => true
				),
				
				array(
					'type' => 'select',
					'label'    => $this->l( 'Complete' ),
					'name'     => 'FORGINGBLOCK_COMPLETEST',										
					'options' => array(
                            'query' => $states,
                            'id' => 'id_order_state',
                            'name' => 'name',
                        ),					
					'required' => true
				),
				array(
					'type' => 'select',
					'label'    => $this->l( 'Invalid' ),
					'name'     => 'FORGINGBLOCK_INVALIDST',										
					'options' => array(
                            'query' => $states,
                            'id' => 'id_order_state',
                            'name' => 'name',
                        ),					
					'required' => true
				),
				array(
					'type' => 'select',
					'label'    => $this->l( 'Expired' ),
					'name'     => 'FORGINGBLOCK_EXPIREDST',										
					'options' => array(
                            'query' => $states,
                            'id' => 'id_order_state',
                            'name' => 'name',
                        ),					
					'required' => true
				),
				
				array(
					'type' => 'select',
					'label'    => $this->l( 'Paid after expiration' ),
					'name'     => 'FORGINGBLOCK_PAIDEXPIREDST',										
					'options' => array(
                            'query' => $states,
                            'id' => 'id_order_state',
                            'name' => 'name',
                        ),					
					'required' => true
				),
				array(
					'type' => 'select',
					'label'    => $this->l( 'Expired with partial payment' ),
					'name'     => 'FORGINGBLOCK_PARTEXPIREDST',										
					'options' => array(
                            'query' => $states,
                            'id' => 'id_order_state',
                            'name' => 'name',
                        ),					
					'required' => true
				),
				
				
			),
			'submit' => array(
				'title' => $this->l( 'Save' ),
				'class' => 'btn btn-default pull-right'
			)
		);
		

		$helper = new HelperForm();

		// Module, token and currentIndex
		$helper->module          = $this;
		$helper->name_controller = $this->name;
		$helper->token           = Tools::getAdminTokenLite( 'AdminModules' );
		$helper->currentIndex    = AdminController::$currentIndex . '&configure=' . $this->name;

		// Language
		$helper->default_form_language    = $default_lang;
		$helper->allow_employee_form_lang = $default_lang;

		// Title and toolbar
		$helper->title          = $this->displayName;
		$helper->show_toolbar   = true;        // false -> remove toolbar
		$helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
		$helper->submit_action  = 'submit' . $this->name;
		$helper->toolbar_btn    = array(
			'save' =>
				array(
					'desc' => $this->l( 'Save' ),
					'href' => AdminController::$currentIndex . '&configure=' . $this->name . '&save' . $this->name .
					          '&token=' . Tools::getAdminTokenLite( 'AdminModules' ),
				),
			'back' => array(
				'href' => AdminController::$currentIndex . '&token=' . Tools::getAdminTokenLite( 'AdminModules' ),
				'desc' => $this->l( 'Back to list' )
			)
		);

		// Load current value
		$helper->fields_value['FORGINGBLOCK_TITLE']       = Configuration::get( 'FORGINGBLOCK_TITLE' );
		$helper->fields_value['FORGINGBLOCK_PAIRCODE']  = Configuration::get( 'FORGINGBLOCK_PAIRCODE' );
		$helper->fields_value['FORGINGBLOCK_APIBASEURL']  = Configuration::get( 'FORGINGBLOCK_APIBASEURL' );		
		
		$helper->fields_value['FORGINGBLOCK_TRANSSPD']  = Configuration::get( 'FORGINGBLOCK_TRANSSPD' );	
		
		$helper->fields_value['FORGINGBLOCK_NEWST']  = empty(Configuration::get( 'FORGINGBLOCK_NEWST' )) ? 3 : Configuration::get( 'FORGINGBLOCK_NEWST' );		
		$helper->fields_value['FORGINGBLOCK_PAIDST']  = empty(Configuration::get( 'FORGINGBLOCK_PAIDST' )) ? 11 : Configuration::get( 'FORGINGBLOCK_PAIDST' );				
		$helper->fields_value['FORGINGBLOCK_CONFIRMRFST']  = empty(Configuration::get( 'FORGINGBLOCK_CONFIRMRFST' )) ? 2 : Configuration::get( 'FORGINGBLOCK_CONFIRMRFST' );			
		$helper->fields_value['FORGINGBLOCK_COMPLETEST']  = empty(Configuration::get( 'FORGINGBLOCK_COMPLETEST' )) ? 2 : Configuration::get( 'FORGINGBLOCK_COMPLETEST' );			
		$helper->fields_value['FORGINGBLOCK_INVALIDST']  = empty(Configuration::get( 'FORGINGBLOCK_INVALIDST' )) ? 8 : Configuration::get( 'FORGINGBLOCK_INVALIDST' );		
		$helper->fields_value['FORGINGBLOCK_EXPIREDST']  = empty(Configuration::get( 'FORGINGBLOCK_EXPIREDST' )) ? 8 : Configuration::get( 'FORGINGBLOCK_EXPIREDST' );				
		$helper->fields_value['FORGINGBLOCK_PAIDEXPIREDST']  = empty(Configuration::get( 'FORGINGBLOCK_PAIDEXPIREDST' )) ? 8 : Configuration::get( 'FORGINGBLOCK_PAIDEXPIREDST' );
		$helper->fields_value['FORGINGBLOCK_PARTEXPIREDST']  = empty(Configuration::get( 'FORGINGBLOCK_PARTEXPIREDST' )) ? 8 : Configuration::get( 'FORGINGBLOCK_PARTEXPIREDST' );		

		

		return $helper->generateForm( $fields_form );
	}	
	
    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }

        $payment_options = [
            $this->ForgingblockExternalPaymentOption(),
        ];
        return $payment_options;
    }	

   
	

    public function ForgingblockExternalPaymentOption()
    {		
		if (isset($_GET['ForgingblockPaymenterror'])) $errmsg = $_GET['ForgingblockPaymenterror'];	
		
		$this->context->smarty->assign( array(
			'this_path'     => $this->_path,
			'this_path_bw'  => $this->_path,
            'errmsg' => $errmsg,									
			'this_path_ssl' => Tools::getShopDomainSsl( true, true ) . __PS_BASE_URI__ . 'modules/' . $this->name . '/',
			'forgingblock_title'  => Configuration::get( 'FORGINGBLOCK_TITLE' )
		) );

		

		$paymentController = $this->context->link->getModuleLink($this->name,'payment',array(),true);

        $newOption = new PaymentOption();
        $newOption->setCallToActionText($this->l('Pay with Forgingblock'))
			->setAction($paymentController)
            ->setAdditionalInformation($this->context->smarty->fetch('module:forgingblock/views/templates/front/payment_infos.tpl'));

        return $newOption;
    }

	
		
    public static function setOrderStatus($oid, $status)
    {
		
		$order = new Order((int)Order::getOrderByCartId($oid));
		$new_history = new OrderHistory();
        $new_history->id_order = (int)$order->id;        
        $new_history->changeIdOrderState((int)$status, $order, true);
        $new_history->addWithemail(true);
		  
    }
	



}
